/*
	Copyright (C) 2004-2005 Cory Nelson

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/08/27 12:15:01 $
		$Revision: 1.1 $
*/

#include <cstring>
#include <iostream>
#include <stdexcept>
#include <p2p/list.hpp>
#ifdef _WIN32
#include <windows.h>
#endif
#include <mysql.h>
#include "../libp2p/src/utf8.h"
#include "config.h" // defines IPDB_SERVER, IPDB_DATABASE, IPDB_USERNAME, IPDB_PASSWORD strings.
using namespace std;

int main(int argc, char *argv[]) {
	if(argc<7) {
		cerr
			<< "usage: " << argv[0] << " [category] [p2p|p2b|p2b1|p2b2|p2b3] [IPDB_SERVER] [IPDB_DATABASE] [IPDB_USERNAME] [IPDB_PASSWORD] [outfile]" << endl
			<< "if outfile is not specified, writes to stdout" << endl;

		return -1;
	}

	p2p::list::file_type type;
	if(!strcmp(argv[2], "p2p")) type=p2p::list::file_p2p;
	else if(!strcmp(argv[2], "p2b")) type=p2p::list::file_p2b;
	else if(!strcmp(argv[2], "p2b1")) type=p2p::list::file_p2b1;
	else if(!strcmp(argv[2], "p2b2")) type=p2p::list::file_p2b2;
	else if(!strcmp(argv[2], "p2b3")) type=p2p::list::file_p2b3;
	else {
		cerr << "unknown output type" << endl;
		return -1;
	}

        char *IPDB_SERVER=argv[3];
        char *IPDB_DATABASE=argv[4];
        char *IPDB_USERNAME=argv[5];
        char *IPDB_PASSWORD=argv[6];

	p2p::list l;

	MYSQL *con=mysql_init(NULL);
	if(!con) {
		cerr << "error: unable to initialize mysql" << endl;
		return -1;
	}

	if(mysql_real_connect(con, IPDB_SERVER, IPDB_USERNAME, IPDB_PASSWORD, IPDB_DATABASE, 0, NULL, 0)!=con) {
		mysql_close(con);

		cerr << "error: unable to connect to database" << endl;
		return -1;
	}

	MYSQL_STMT *stmt=mysql_stmt_init(con);
	if(!stmt) {
		mysql_close(con);

		cerr << "error: unable to initialize prepared statement" << endl;
		return -1;
	}

	if(mysql_stmt_prepare(stmt, "select name,start,end from cat_ip,ips,groups where category=? and blocked='blocked' and start=ip and ID=groupID", 111)!=0) {
		mysql_stmt_close(stmt);
		mysql_close(con);

		cerr << "error: unable to prepare statement" << endl;
		return -1;
	}

	MYSQL_BIND param={0};
	unsigned long paramlen=(unsigned long)strlen(argv[1]);

	param.buffer_type=MYSQL_TYPE_STRING;
	param.buffer=argv[1];
	param.buffer_length=paramlen;
	param.length=&paramlen;
	param.is_null=NULL;

	if(mysql_stmt_bind_param(stmt, &param)!=0) {
		mysql_stmt_close(stmt);
		mysql_close(con);

		cerr << "error: unable to bind parameter" << endl;
		return -1;
	}

	if(mysql_stmt_execute(stmt)!=0) {
		mysql_stmt_close(stmt);
		mysql_close(con);

		cerr << "error: unable to execute statement" << endl;
		return -1;
	}

	char name[128];
	unsigned long namelen;
	unsigned long startip, endip;
	MYSQL_BIND result[3]={0};

	result[0].buffer_type=MYSQL_TYPE_STRING;
	result[0].buffer=name;
	result[0].buffer_length=128;
	result[0].length=&namelen;
	result[0].is_null=NULL;

	result[1].buffer_type=MYSQL_TYPE_LONG;
	result[1].buffer=(char*)&startip;
	result[1].buffer_length=sizeof(startip);
	result[1].is_unsigned=1;
	result[1].is_null=NULL;

	result[2].buffer_type=MYSQL_TYPE_LONG;
	result[2].buffer=(char*)&endip;
	result[1].buffer_length=sizeof(endip);
	result[2].is_unsigned=1;
	result[2].is_null=NULL;

	if(mysql_stmt_bind_result(stmt, result)!=0) {
		mysql_stmt_close(stmt);
		mysql_close(con);

		cerr << "error: unable to bind results" << endl;
		return -1;
	}

	int ret;
	while((ret=mysql_stmt_fetch(stmt))==0) {
		p2p::range r;

		utf8::utf8_wchar((const char*)name, ((const char*)name)+namelen, back_inserter(r.name));
		r.start=startip;
		r.end=endip;

		l.insert(r);
	}

	mysql_stmt_close(stmt);
	mysql_close(con);

	if(ret!=MYSQL_NO_DATA) {
		cerr << "error: unable to fetch statement" << endl;
		return -1;
	}

	l.optimize();

	try {
		if(argc==8) l.save(argv[7], type);
		else l.save(cout, type);
	}
	catch(const exception &ex) {
		cerr << "error: unable to save: " << ex.what() << endl;
		return -1;
	}

	return 0;
}
