#include <set>
#include <vector>
#include <string>
#include <fstream>
#include <iostream>
#include <boost/regex.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/algorithm/string/regex.hpp>
#include <p2p/list.hpp>
using namespace std;
using namespace boost;

// regex: -\d+-\d+\s+\(NET-\d+-\d+-\d+-\d+-\d+\)
// matches: -113-26 (NET-4-37-113-112-1)

static const wregex g_regex(L"-\\d+-\\d+\\s+\\(NET-\\d+-\\d+-\\d+-\\d+-\\d+\\)");

// regex: (\s+\d+)+$
// matches: 1434 5000

static const wregex g_regex2(L"(\\s+\\d+)+$");

// regex: ^\(\d+\)\s+
// matches: (050510)

static const wregex g_regex3(L"^\\(\\d+\\)\\s+");

static void fix_string(wstring &str) {
	if(str.find(L' ')==wstring::npos) {
		if(str.find_first_of(L"-_")!=wstring::npos) {
			replace_all(str, L"-", L" ");
			replace_all(str, L"_", L" ");
		}
		else replace_all(str, L".", L" ");
	}
}

static void clean_str(wstring &str) {
	erase_all_regex(str, g_regex);
	erase_all_regex(str, g_regex2);
	erase_all_regex(str, g_regex3);
	erase_all(str, " Old Ap2p");

	if(all(str, is_upper()||is_punct()||is_digit()||is_space()||is_cntrl()))
		to_lower(str);

	for(wstring::size_type i=0; i<str.length(); i++)
		if((i==0 || iswspace(str[i-1])) && iswlower(str[i])) str[i]=towupper(str[i]);

	trim(str);

	fix_string(str);
}

static void usage(const char *app) {
	cerr << "usage: " << app << " [-p2p|-p2b] [-o output] <filter> <list>" << endl;
}

int main(int argc, char *argv[]) {
	if(argc<3) {
		usage(argv[0]);
		return -1;
	}

	bool outp2p=true;
	const char *outfile=NULL;

	int i=1;
	for(i=1; i<argc; i++) {
		if(argv[i][0]=='-') {
			if(!strcmp(argv[i]+1, "p2p")) outp2p=true;
			else if(!strcmp(argv[i]+1, "p2b")) outp2p=false;
			else if(!strcmp(argv[i]+1, "o")) {
				if(++i < argc) outfile=argv[i];
				else {
					usage(argv[0]);
					return -1;
				}
			}
		}
		else break;
	}

	if(i+1 >= argc) {
		usage(argv[0]);
		return -1;
	}

	vector<wstring> filter;
	{
		set<wstring> strings;
		{
			wifstream fs(argv[i]);

			if(fs.is_open()) {
				wstring s;
				while(getline(fs, s)) {
					if(s.length()>0) {
						to_lower(s);
						strings.insert(s);
					}
				}
			}
			else {
				cerr << "unable to open file \"" << argv[i] << "\", bailing..." << endl;
				return -1;
			}
		}

		filter.reserve(strings.size());
		filter.assign(strings.begin(), strings.end());
	}

	p2p::list dest;
	{
		p2p::list l;
		try {
			l.load(argv[i+1]);
		}
		catch(exception &ex) {
			cerr << ex.what() << endl;
			return -1;
		}

		for(p2p::list::const_iterator iter=l.begin(); iter!=l.end(); iter++) {
			wstring name=iter->name;
			fix_string(name);
			to_lower(name);

			bool add=true;

			for(vector<wstring>::size_type i=0; i<filter.size(); i++) {
				if(name.find(filter[i])!=wstring::npos) {
					add=false;
					break;
				}
			}

			if(add) {
				p2p::range r=*iter;
				
				fix_string(r.name);
				clean_str(r.name);

				dest.insert(r);
			}
		}
	}

	dest.optimize();
	if(outfile) dest.save(outfile, outp2p?p2p::list::file_p2p:p2p::list::file_p2b);
	else dest.save(cout, p2p::list::file_p2p);

	return 0;
}
